#define HCX_INPUT_WORDLIST		'i'
#define HCX_OUTPUT_DIGITLIST		'd'
#define HCX_OUTPUT_XDIGITLIST		'x'
#define HCX_OUTPUT_CHARLIST		'c'
#define HCX_OUTPUT_CSLIST		's'
#define HCX_HELP			'h'
#define HCX_VERSION			'v'

#define LINEIN_MAX	256
#define LINE_MAX	68
#define WORD_MAX	64

#define DIGITWORD	0
#define XDIGITWORD	1
#define CHARWORD	2
#define CSWORD		3

